/* ccgo: go/arg.hh
 *
 * Copyright 2004 Chun-Chung Chen <cjj@u.washington.edu>
 *
 * This program is free software; you can redistribute it and'or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */

#ifndef GO_ARG_HH
#define GO_ARG_HH

#include <vector>
#include <string>

namespace arg {
	class Value
	{
	public:
		virtual bool set(const std::string & str) = 0;
	};

	class V_int :
		public Value
	{
	public:
		virtual bool set(const std::string & str);
		operator int();
	private:
		int v;
	};

	class V_double :
		public Value
	{
	public:
		virtual bool set(const std::string & str);
		operator double();
	private:
		double v;
	};

	class V_string :
		public Value
	{
	public:
		virtual bool set(const std::string & str);
		operator std::string();
	private:
		std::string v;
	};

	class Parser
	{
	public:
		void add_opt(int key, std::string name, Value * store = 0, bool optional = false);
		bool parse(int argc, char * argv[]); // true: successful, false: unsuccessful
		bool check(int key);
		const std::vector<std::string> & get_args() const;
	protected:
		virtual void new_arg();
		virtual void add_arg(const std::string & str);
	private:
		std::vector<std::string> a; // arguments
		struct Opt
		{
		        int k;
			std::string n;
			Value * s; // storage
			bool o; // optional
			bool p; // present?
		};
		std::vector<Opt> l;
	};
}

#endif // GO_ARG_HH
